﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;

namespace ElfViewerConsole.ELFModel.ELFFormat
{
    class ELFNoteTable
    {
        public ELFSectionHeaderTable SectionHeaderTable { get; private set; }
        private ELFFile ElfFile;

        private List<ELFNote> noteEntries;

        public ELFNoteTable(ELFFile elfFile, ELFSectionHeaderTable secTableHeader)
        {
            this.SectionHeaderTable = secTableHeader;
            this.ElfFile = elfFile;
        }

        internal void Read()
        {
            uint backupFilePos = this.ElfFile.FileData.Position;

            noteEntries = new List<ELFNote>();

            this.ElfFile.FileData.Position = (uint)SectionHeaderTable.Sh_Offset.Value;

            while (ElfFile.FileData.Position + 0x18 <= SectionHeaderTable.Sh_Offset.Value + SectionHeaderTable.Sh_Size.Value)
            {
                uint namesz = this.ElfFile.FileData.ReadUInt32();
                uint descsz = this.ElfFile.FileData.ReadUInt32();

                uint Type = this.ElfFile.FileData.ReadUInt32();

                string Name = String.Empty;
                if (namesz > 0)
                {
                    Name = this.ElfFile.FileData.ReadAscii(this.ElfFile.FileData.Position, (int)namesz);
                    this.ElfFile.FileData.Position += namesz;
                    while ((namesz & 0x03) != 0)
                    {
                        this.ElfFile.FileData.Position++;
                        namesz++;
                    }
                }

                ELFElemByteArray Description;
                if (descsz > 0)
                {
                    Description = new ELFElemByteArray(this.ElfFile.FileData.Position, this.ElfFile.FileData.ReadByteArray(descsz), descsz, "Desc");
                    while ((descsz & 0x03) != 0)
                    {
                        this.ElfFile.FileData.Position++;
                        descsz++;
                    }
                }
                else
                {
                    Description = new ELFElemByteArray(0, new byte[1], 1, "Empty");
                }

                noteEntries.Add(new ELFNote(Name, Type, Description));
            }

            this.ElfFile.FileData.Position = backupFilePos;
        }

        public IEnumerator GetEnumerator()
        {
            foreach (ELFNote elfNote in noteEntries)
            {
                yield return elfNote;
            }
        }
    }
}
