﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ELFViewerConsole.ELFModel;
using ElfViewerConsole.ELFModel.ELFFormat.Enums;
using System.Collections;

namespace ElfViewerConsole.ELFModel.ELFFormat
{
    class ELFProgramHeaderTable : IDisposable
    {
        private ELFFile ElfFile;

        public int HeaderTableIndex { get; private set; }

        public ELFElem32Enum<EELFPType> P_Type { get; private set; }
        public ELFElem32Enum<EELFPFlags> P_Flags { get; private set; }
        public ELFElem64 P_Off { get; private set; }
        public ELFElem64 P_VAddr { get; private set; }
        public ELFElem64 P_PAddr { get; private set; }
        public ELFElem64 P_FileSz { get; private set; }
        public ELFElem64 P_MemSz { get; private set; }
        public ELFElem64 P_Align { get; private set; }

        public ELFProgramHeaderTable(int index, ELFFile elfFile)
        {
            this.ElfFile = elfFile;
            this.HeaderTableIndex = index;
        }

        public void Dispose()
        {
            /* nothing to do */
        }

        internal void Read(IFileAccess fDataReader)
        {
            uint filePos = 0;

            try
            {
                filePos = fDataReader.Position;

                /* move to start of program header table */
                fDataReader.Position = (uint)(ElfFile.ElfFileHeader.E_Phoff.Value + (ulong)(HeaderTableIndex * ElfFile.ElfFileHeader.E_PhEntSize.Value));

                if (ElfFile.Is64Bit())
                {
                    P_Type = new ELFElem32Enum<EELFPType>(fDataReader.Position, fDataReader.ReadUInt32(), "P_TYPE");
                    P_Flags = new ELFElem32Enum<EELFPFlags>(fDataReader.Position, fDataReader.ReadUInt32(), "P_FLAGS");
                    P_Off = new ELFElem64(fDataReader.Position, fDataReader.ReadUInt64(), "P_OFF");
                    P_VAddr = new ELFElem64(fDataReader.Position, fDataReader.ReadUInt64(), "P_VADDR");
                    P_PAddr = new ELFElem64(fDataReader.Position, fDataReader.ReadUInt64(), "P_PADDR");
                    P_FileSz = new ELFElem64(fDataReader.Position, fDataReader.ReadUInt64(), "P_FILESZ");
                    P_MemSz = new ELFElem64(fDataReader.Position, fDataReader.ReadUInt64(), "P_MEMSZ");
                    P_Align = new ELFElem64(fDataReader.Position, fDataReader.ReadUInt64(), "P_ALIGN");
                }
                else
                {
                    P_Type = new ELFElem32Enum<EELFPType>(fDataReader.Position, fDataReader.ReadUInt32(), "P_TYPE");
                    P_Off = new ELFElem64(fDataReader.Position, fDataReader.ReadUInt32(), "P_OFF", 4);
                    P_VAddr = new ELFElem64(fDataReader.Position, fDataReader.ReadUInt32(), "P_VADDR", 4);
                    P_PAddr = new ELFElem64(fDataReader.Position, fDataReader.ReadUInt32(), "P_PADDR", 4);
                    P_FileSz = new ELFElem64(fDataReader.Position, fDataReader.ReadUInt32(), "P_FILESZ", 4);
                    P_MemSz = new ELFElem64(fDataReader.Position, fDataReader.ReadUInt32(), "P_MEMSZ", 4);
                    P_Flags = new ELFElem32Enum<EELFPFlags>(fDataReader.Position, fDataReader.ReadUInt32(), "P_FLAGS");
                    P_Align = new ELFElem64(fDataReader.Position, fDataReader.ReadUInt32(), "P_ALIGN", 4);
                }
            }
            finally
            {
                fDataReader.Position = filePos;
            }
        }

        public IEnumerator GetEnumerator()
        {
            yield return P_Type;
            yield return P_Flags;
            yield return P_Off;
            yield return P_VAddr;
            yield return P_PAddr;
            yield return P_FileSz;
            yield return P_MemSz;
            yield return P_Align;
        }
    }
}
