﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ElfViewerConsole.ELFModel.ELFFormat
{
    class ELFSymbolTable
    {
        public ELFSectionHeaderTable SectionHeaderTable { get; private set; }
        private ELFFile ElfFile;
        private List<ELFSymbolTableEntry> SymbolEntries;

        public ELFSymbolTable(ELFFile elfFile, ELFSectionHeaderTable secTableHeader)
        {
            this.SectionHeaderTable = secTableHeader;
            this.ElfFile = elfFile;
        }

        internal void Read()
        {
            int numOfEntrys = (int)(SectionHeaderTable.Sh_Size.Value / SectionHeaderTable.Sh_Entsize.Value);
            SymbolEntries = new List<ELFSymbolTableEntry>();
            for (int i = 0; i < numOfEntrys; i++)
            {
                ELFSymbolTableEntry tblEntry = new ELFSymbolTableEntry(i, this.ElfFile, this);
                tblEntry.Read(this.ElfFile.FileData, (uint)((uint)SectionHeaderTable.Sh_Offset.Value + (i * (uint)SectionHeaderTable.Sh_Entsize.Value)));
                SymbolEntries.Add(tblEntry);
            }
        }

        public IEnumerable<ELFSymbolTableEntry> GetSymbolTableEntries()
        {
            if (SymbolEntries != null)
            {
                foreach (ELFSymbolTableEntry symTabEntry in this.SymbolEntries)
                {
                    yield return symTabEntry;
                }
            }
        }

        internal ELFSectionHeaderTable GetLinkedStringTable()
        {
            return ElfFile.SectionHeaderTables[(int)(SectionHeaderTable.Sh_Link.Value)];
        }

        internal uint GetLinkedStringTableFileOffset()
        {
            return (uint)ElfFile.SectionHeaderTables[(int)(SectionHeaderTable.Sh_Link.Value)].Sh_Offset.Value;
        }


    }
}
