﻿/**
 * Sunshine's ELF Viewer
 * ---------------------
 * Version 1.0 / 2018-10-01
 *
 * Simple console application to read the most important information from 32/64 bit ELF files.
 * 
 * Supports ELF header, section header table, program header table, symbol tables and note table.
 * Each table information can be shown in short and detailed mode.
 * 
 * Does not (yet?) support dynamic tables, hash tables and relocation tables.
 * 
 * www.sunshine2k.de || www.bastian-molkenthin.de
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ElfViewerConsole.ELFModel.ELFFormat;
using System.IO;
using ElfViewerConsole.ELFModel.ELFFormat.Enums;
using ElfViewerConsole;

namespace ELFViewerConsole
{
    class Program
    {
        static void Main(string[] args)
        {

#if DEBUG_
            Console.WindowWidth += 50;
            Console.BufferHeight += 3000;

            ELFFile elffile = new ELFFile("test.elf");
            elffile.LoadFile();

            ConsoleOutput co = new ConsoleOutput(elffile);
            //co.PrintGeneralFileInfo();

            co.PrintElfHeaderDetailed();
            //co.PrintElfHeaderShort();


            if (elffile.ExistsProgramHeaderTable())
            {
                //co.PrintProgramHeaderTableDetailed();
                co.PrintProgramHeaderTableShort();
            }


            if (elffile.ExistsSectionHeaderTables())
            {
                //co.PrintSectionHeaderTableDetailed();
                  co.PrintSectionHeaderTableShort();
            }

            if (elffile.ExistsSymbolTables())
            {
                //co.PrintSymbolTablesDetailed();
                co.PrintSymbolTablesShort();
            }

            co.PrintNoteTables();
#else
            CmdLineArgs cmdArgs = new CmdLineArgs();
            CmdLineArgs.CMDLINEARGS_RESULT cmdRes = cmdArgs.ProcessArgs(args);
            if (cmdRes == CmdLineArgs.CMDLINEARGS_RESULT.CMDLINEARGS_NONE)
            {
                cmdArgs.PrintUsage();
            }
            else if (cmdRes == CmdLineArgs.CMDLINEARGS_RESULT.CMDLINEARGS_FILENOTFOUND)
            {
                Console.Out.WriteLine("Specified file does not exist: " + cmdArgs.FileName);
                Console.Out.WriteLine("Bye...");
            }
            else if (cmdRes == CmdLineArgs.CMDLINEARGS_RESULT.CMDLINEARGS_INVALID)
            {
                Console.Out.WriteLine("Invalid argument specified: " + cmdArgs.InvalidArg);
                Console.Out.WriteLine("Bye...");
            }
            else
            {
                ShowElfFileInfo(cmdArgs);
            }
#endif


#if DEBUG
            Console.In.ReadLine();
#endif
        }

        static void ShowElfFileInfo(CmdLineArgs cmdArgs)
        {
            ELFFile elffile = new ELFFile(cmdArgs.FileName);

            try
            {
                elffile.LoadFile();
            }
            catch (Exception ex)
            {
                Console.Out.WriteLine("Error occured during file loading: \n" + ex.StackTrace);
                return;
            }

            if (!elffile.Valid)
            {
                Console.Out.WriteLine("File is not a valid ELF file. Aborting...");
                return;
            }

            ConsoleOutput co = new ConsoleOutput(elffile);

            Dictionary<string, CmdLineArgs.CmdLineOpt> opts = cmdArgs.GetOptions();
            if (opts[CmdLineArgs.OptStrGeneralFileInfo].active)
            {
                co.PrintGeneralFileInfo();
            }

            if (opts[CmdLineArgs.OptStrELFHeaderShort].active)
            {
                co.PrintElfHeaderShort();
            }
            if (opts[CmdLineArgs.OptStrELFHeaderDetailed].active)
            {
                co.PrintElfHeaderDetailed();
            }

            if (opts[CmdLineArgs.OptStrProgramHeaderTableShort].active)
            {
                co.PrintProgramHeaderTableShort();
            }
            if (opts[CmdLineArgs.OptStrProgramHeaderTableDetailed].active)
            {
                co.PrintProgramHeaderTableDetailed();
            }

            if (opts[CmdLineArgs.OptStrSectionHeaderTableShort].active)
            {
                co.PrintSectionHeaderTableShort();
            }
            if (opts[CmdLineArgs.OptStrSectionHeaderTableDetailed].active)
            {
                co.PrintSectionHeaderTableDetailed();
            }

            if (opts[CmdLineArgs.OptStrSymbolTablesShort].active)
            {
                co.PrintSymbolTablesShort();
            }
            if (opts[CmdLineArgs.OptStrSymbolTablesDetailed].active)
            {
                co.PrintSymbolTablesDetailed();
            }

            if (opts[CmdLineArgs.OptStrNoteTables].active)
            {
                co.PrintNoteTables();
            }
         }

    }
}
